@@ -52,7 +52,9 @@ namespace libsemigroups {
   class FpSemigroupInterface : public Runner {
    public:
     //! Type for rules in a finitely presented semigroup.
-    using rule_type = std::pair<std::string, std::string>;
+    using char_type   = char;
+    using string_type = std::string;
+    using rule_type   = std::pair<string_type, string_type>;
 
     //! Type for const iterators to the defining rules of the finitely
     //! presented semigroup represented by \c this.
@@ -112,7 +114,7 @@ namespace libsemigroups {
     //!
     //! \par Parameters
     //! (None)
-    virtual size_t size() = 0;
+    virtual uint64_t size() = 0;
 
     //! Check if two words represent the same element.
     //!
@@ -376,6 +378,10 @@ namespace libsemigroups {
       set_identity(std::string(1, _alphabet[id]));
     }
 
+    bool has_identity() const noexcept {
+      return _identity_defined;
+    }
+
     //! Set the inverses of letters in alphabet().
     //!
     //! The letter in \p a with index \c i is the inverse of the letter in
@@ -512,6 +518,21 @@ namespace libsemigroups {
       return _alphabet;
     }
 
+    //! Returns the i-th letter of the alphabet of the finitely presented
+    //! semigroup represented by \p this.
+    //!
+    //! \param i the index of the letter.
+    //!
+    //! \returns A std::string by value.
+    //!
+    //! \throws std::range_error if the index \p i is out of range.
+    //!
+    //! \complexity
+    //! Constant.
+    std::string alphabet(size_t i) const {
+      return std::string({_alphabet.at(i)});
+    }
+
     //! Returns the identity of \p this, or throws an exception if there isn't
     //! one.
     //!
