@@ -29,6 +29,7 @@
 
 #include "function-ref.hpp"             // for FunctionRef
 #include "libsemigroups-exception.hpp"  // for LibsemigroupsException
+#include "report.hpp"                   // for REPORT_DEFAULT
 
 namespace libsemigroups {
   //! A pseudonym for std::chrono::nanoseconds::max().
@@ -88,12 +89,18 @@ namespace libsemigroups {
     //! is the same as \p copy, except that the function passed as an argument
     //! to run_until (if any) is not copied.
     //!
-    //! \param copy the Runner to copy.
+    //! \param other the Runner to copy.
     Runner(Runner const& other) : Runner() {
       _state = other._state.load();
     }
 
+    //! Move constructor
     //!
+    //! Returns a runner that is initialised from \p other. The state of the
+    //! new runner is the same as \p copy, except that the function passed as
+    //! an argument to run_until (if any) is not copied.
+    //!
+    //! \param other the Runner to move from.
     Runner(Runner&& other) : Runner() {
       _state = other._state.load();
     }
@@ -205,6 +212,7 @@ namespace libsemigroups {
     // At the end of this either finished, dead, or stopped_by_predicate.
     template <typename T>
     void run_until(T&& func) {
+      REPORT_DEFAULT("running until predicate returns true or finished. . .\n");
       if (!finished() && !dead()) {
         before_run();
         _stopper = std::forward<T>(func);
